<?php

namespace FiloBlu\CheckoutExtend\Controller\Index;


use FiloBlu\CheckoutExtend\Helper\Data;
use FiloBlu\CheckoutExtend\Model\Vat;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\Result\JsonFactory;

class ValidateVat extends \Magento\Framework\App\Action\Action
{

    /**
     * @var JsonFactory
     */
    protected $_resultJsonFactory;
    /**
     * @var Data
     */
    protected $helperData;
    /**
     * @var Vat
     */
    private $vat;
    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $checkoutSession;
    protected $context;
    /**
     * Application Event Dispatcher
     *
     * @var \Magento\Framework\Event\ManagerInterface
     */
    protected $eventManager;
    /**
     * @var \Magento\Quote\Model\Quote\TotalsCollector
     */
    private $totalsCollector;
    /**
     * @var \Magento\Quote\Observer\Frontend\Quote\Address\VatValidator
     */
    protected $vatValidator;

    /**
     * ValidateVat constructor.
     * @param Context $context
     * @param JsonFactory $resultJsonFactory
     * @param Data $helperData
     * @param Vat $vat
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Framework\Event\ManagerInterface $eventManager
     * @param \Magento\Quote\Model\Quote\TotalsCollector $totalsCollector
     * @param \Magento\Quote\Observer\Frontend\Quote\Address\VatValidator $vatValidator
     */
    public function __construct(
        Context $context,
        JsonFactory $resultJsonFactory,
        Data $helperData,
        Vat $vat,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Framework\Event\ManagerInterface $eventManager,
        \Magento\Quote\Model\Quote\TotalsCollector $totalsCollector,
        \Magento\Quote\Observer\Frontend\Quote\Address\VatValidator $vatValidator

    )
    {
        $this->helperData = $helperData;
        $this->_resultJsonFactory = $resultJsonFactory;
        parent::__construct($context);
        $this->vat = $vat;
        $this->checkoutSession = $checkoutSession;
        $this->eventManager = $eventManager;
        $this->totalsCollector = $totalsCollector;
        $this->vatValidator = $vatValidator;

    }

    public function execute()
    {

        $vatConfig = $this->helperData->getCustomerGroupVatSeparation();
        if ($this->getRequest()->isXmlHttpRequest() || $this->getRequest()->getParam('ajax') || $this->getRequest()->getParam('isAjax')) {
            $resultValue = $this->getResultIsValid();
            $this->getDispatchEvent();
            $resultJson = $this->_resultJsonFactory->create();
            return $resultJson->setData([
                'success' => $resultValue
            ]);
        }
        return false;

    }

    public function getResultIsValid()
    {
        $data = $this->getRequest()->getParam('vat-number');
        $result = $this->vat->checkVatNumber($this->getRequest()->getCookie('filoblu_geoip_country'), $data, $this->vat->getMerchantCountryCode(), $this->vat->getMerchantVatNumber());
        $resultValue = $result->getIsValid();
        return $resultValue;
    }

    public function getDispatchEvent()
    {
//        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
//        /** @var \Magento\Quote\Observer\Frontend\Quote\Address\VatValidator $validator */
//        $validator = $objectManager->get('\Magento\Quote\Observer\Frontend\Quote\Address\VatValidator');
        $validator = $this->vatValidator;
        $this->checkoutSession->getQuote()->getShippingAddress()->setVatId($this->getRequest()->getParam('vat-number'));
        $resultValidator = $validator->validate($this->checkoutSession->getQuote()->getShippingAddress(), $this->checkoutSession->getQuote()->getStore());

        $this->vat->getCustomerGroupIdBasedOnVatNumber($this->getRequest()->getCookie('filoblu_geoip_country'), $resultValidator, $this->checkoutSession->getQuote()->getStoreId());

        $this->eventManager->dispatch(
            'sales_quote_collect_totals_after',
            [
                'quote' => $this->checkoutSession->getQuote()
            ]
        );
    }
}
